<#
.SYNOPSIS
   <A brief description of the script>
.DESCRIPTION
   <A detailed description of the script>
.PARAMETER <paramName>
   <Description of script parameter>
.EXAMPLE
   <An example of using the script>
#>

function Copy-CSVtoSheet{
	param($to, $CSVname)

	$csvBook = $excelApp.Workbooks.Open($CSVname)
	$csvSheet = $csvBook.Worksheets.Item(1)
	$csvSheet.UsedRange.Copy() | Out-Null
	$to.Paste()
	$to.UsedRange.EntireColumn.AutoFit() | Out-Null
	$csvbook.Application.CutCopyMode = $false
	$csvBook.Close($false,$null,$null)
}


$xDays = 60
$Date = Get-Date -uformat %d-%m-%Y
$vCenter = Connect-VIServer -Server 
	

$tempCSV = $env:Temp + "\Report-" + (Get-Date).Ticks + ".csv"

$excelApp = New-Object -ComObject "Excel.Application"
$workBook = $excelApp.Workbooks.Add()

# Report 1: vCenter report
$sheet = $excelApp.Worksheets.Add()
$sheet.Name = "vCenter"

#$report | Export-Csv -Path $tempCSV -NoTypeInformation
#Copy-CSVtoSheet $sheet $tempCSV

# Report 2: Cluster report
$sheet = $excelApp.Worksheets.Add()
$sheet.Name = "Clusters"

&{Get-Cluster |
ForEach-Object {
	$Cluster = $_
	$VMHost = $Cluster | Get-VMHost
	$CpuUsageMhz = $VMHost | Measure-Object -Property CpuUsageMhz -Sum | Select-Object -ExpandProperty Sum
	$CpuTotalMhz = $VMHost | Measure-Object -Property CpuTotalMhz -Sum | Select-Object -ExpandProperty Sum
	$MemoryUsageGB = $VMHost | Measure-Object -Property MemoryUsageGB -Sum | Select-Object -ExpandProperty Sum
	$MemoryTotalGB = $VMHost | Measure-Object -Property MemoryTotalGB -Sum | Select-Object -ExpandProperty Sum
	$Cluster | Select-Object -Property @{Name="ClusterName";Expression={$_.Name}},
	@{Name="vSphere HA";Expression={$_.HAEnabled}},
	@{Name="HA-Host Monitoring";Expression={$_.ExtensionData.Configuration.DasConfig.HostMonitoring}},
	@{Name="HA-Failover Capacity";Expression={$_.ExtensionData.Summary.CurrentFailoverLevel}},
	@{Name="HA-Admissions Control";Expression={$_.HAAdmissionControlEnabled}}, 
	@{Name="Configuration Issue";Expression={[string]::Join(',',($_.ExtensionData.ConfigIssue | Select -ExpandProperty FullFormattedMessage))}},
	@{Name="vSphere DRS";Expression={$_.DrsEnabled}},
	@{Name="DRS Automation Level";Expression={$_.DrsAutomationLevel}},
	#@{Name="Cpu Total(Mhz)";Expression={$CpuTotalMhz}},
	#@{Name="Cpu Usage (Mhz)";Expression={$CpuUsageMhz}},
	@{Name="Cpu Usage(%)";Expression={[math]::Round(100 * $CpuUsageMhz / $CpuTotalMhz)}},
	@{Name="Memory Total(GB)";Expression={$MemoryTotalGB}},
	@{Name="Memory Usage(GB)";Expression={$MemoryUsageGB}},	
	#@{Name="Memory Usage (%)";Expression={[math]::Round(100 * $MemoryUsageGB / $MemoryTotalGB)}},
	@{Name="Num of VMs";Expression={$_ | Get-VM | Measure-Object | Select-Object -ExpandProperty Count}},
	@{Name="Num of VMs without VMXNET3";Expression={$_ | Get-VM | Get-NetworkAdapter | Where-object {$_.Type -ne "Vmxnet3"} | Measure-Object | Select-Object -ExpandProperty Count}},
	@{Name="Num of vMotions";Expression={
		Get-VIEvent -Entity (Get-VM -Location $_) -MaxSamples ([int]::MaxValue) -Start (Get-Date).AddDays(-10) | 
		Where { $_.GetType().Name -eq "TaskEvent" -and $_.Info.DescriptionId -eq "VirtualMachine.migrate"} | 
		Measure-Object | Select-Object -ExpandProperty Count}},
	@{Name="Num of SvMotions";Expression={
		Get-VIEvent  -Entity (Get-VM -Location $_) -MaxSamples ([int]::MaxValue) -Start (Get-Date).AddDays(-10) | 
		Where {$_.GetType().Name -eq "TaskEvent" -and $_.Info.DescriptionId -eq "VirtualMachine.relocate"} | 
		Measure-Object | Select-Object -ExpandProperty Count}} 
}} |
Export-Csv -Path $tempCSV -NoTypeInformation -UseCulture

Copy-CSVtoSheet $sheet $tempCSV

# Report 3: Hosts report
$sheet = $excelApp.Worksheets.Add()
$sheet.Name = "Hosts"

Get-VMHost |
Select-Object -Property @{Name="ESXi HostName";Expression={$_.Name}},
      @{Name="Disconnected";Expression={$_.ConnectionState}},
      @{Name="Maintenace Mode";Expression={$_.ExtensionData.Runtime.InMaintenanceMode}},
      @{Name ="BIOS Version";Expression={$_.ExtensionData.Hardware.BiosInfo.BiosVersion}},
      @{Name ="No of Vm's in Swap Datasore";Expression={$_.VMSwapfileDatastore.ExtensionData.Vm.Count}},
      @{Name="Uptime above $xDays days";E={if((New-TimeSpan -Start $_.Extensiondata.Runtime.BootTime.ToLocalTime() -End $Date).Days -gt $xDays){"Yes"}else{"No"}}} |

Export-Csv -Path $tempCSV -NoTypeInformation
Copy-CSVtoSheet $sheet $tempCSV

# Report 4: Datastore Report
$sheet = $excelApp.Worksheets.Add()
$sheet.Name = "Datastores"

$report = @()
## added selected properties to help speed this up, and lower memory usage
$arrUsedDisks = Get-View -ViewType VirtualMachine -Property Name,Layout | % {$_.Layout} | % {$_.Disk} | % {$_.DiskFile}

## removed the "get all hosts, do this search for all hosts"; just need to do this for all datastores, not for all datastores per host

## moved the "make new seach spec" outside of the foreach loops, as this needs only to happen one time
$fileQueryFlags = New-Object VMware.Vim.FileQueryFlags
$fileQueryFlags.FileSize = $true
$fileQueryFlags.FileType = $true
$fileQueryFlags.Modification = $true
$searchSpec = New-Object VMware.Vim.HostDatastoreBrowserSearchSpec
$searchSpec.details = $fileQueryFlags
$searchSpec.matchPattern = "*.vmdk"
$searchSpec.sortFoldersFirst = $true

## removed extraneous things so that this only runs once per datastore
Get-View -ViewType Datastore -Property Name,Browser | %{
    $ds = $_
    $dsBrowser = Get-View $ds.browser
    $rootPath = "[" + $ds.Name + "]"
    $searchResult = $dsBrowser.SearchDatastoreSubFolders($rootPath, $searchSpec)
 
    foreach ($folder in $searchResult) {
        foreach ($fileResult in $folder.File) {
            if ($fileResult.Path) {
                $pathAsString = out-string -InputObject $FileResult.Path
                # Changed block Tracking creates ctk.vmdk files that are not referenced in the VMX.  This prevents them from showing as false positives
                if (-not ($arrUsedDisks -contains ($folder.FolderPath + $fileResult.Path)) -and (-not ($pathAsString.toLower().contains("-ctk.vmdk")))){
                    $report += New-Object -TypeName PSObject -Property @{
                        DS = $ds.Name
                        Path = $folder.FolderPath
                        File = $fileResult.Path
                        SizeGB = $fileResult.FileSize/1MB
                        ModDate = $fileResult.Modification
                    } ## end new-object
                }
            }
        }
    }
}

Get-Datastore | Select-Object -Property Name, CapacityGB, FreeSpaceGB,
    @{ Name = "ProvisionedSpaceGB"; Expression = {($_.ExtensionData.Summary.Capacity - $_.Extensiondata.Summary.FreeSpace + $_.ExtensionData.Summary.Uncommitted)/1GB}},
    ## property with value of $true or $false, based on whether $report has an object with this datastore name
    @{Name="Orphaned VMDKs?"; Expression={($report | %{$_.DS}) -contains $_.Name}}	|
  
Export-Csv -Path $tempCSV -NoTypeInformation
Copy-CSVtoSheet $sheet $tempCSV

# Report 5: VMs Report
$sheet = $excelApp.Worksheets.Add()
$sheet.Name = "VMs"

$report = @()

$vms = Get-VM

$report += New-Object PSObject -Property @{
  "VM Attribute" = "VM's with Active Snapshots" 
  "Specification" = &{
    $result = $vms | where {$_ | Get-Snapshot} | %{$_.Name}
    if($result){
      [string]::Join(',',$result)
    }
	else{"No VMs with Active Snapshot"}
  }
}

$report += New-Object PSObject -Property @{
  "VM Attribute" = "VMware tools outdated" 
  "Specification" = &{
    $result = $vms | where {$_.ExtensionData.Guest.ToolsStatus -eq "toolsOld"} | %{$_.Name}
    if($result){
      [string]::Join(',',$result)
    }
	else{"No VMs with Tools Outdated"}
  }
}

$report += New-Object PSObject -Property @{
  "VM Attribute" = "VM's with CD-ROM Connected" 
  "Specification" = &{
    $result = $vms | where {$_ | Get-CDDrive | where { $_.ConnectionState.Connected -eq "true"}} | %{$_.Name}
    if($result){
      [string]::Join(',',$result)
    }
	else{"No VMs with CD-ROM Detected"}
  }
}

$report += New-Object PSObject -Property @{
  "VM Attribute" = "VM's with Floppy Connected" 
  "Specification" = &{
    $result = $vms | where {$_ | Get-FloppyDrive | where { $_.ConnectionState.Connected -eq "true"}} | %{$_.Name}
    if($result){
      [string]::Join(',',$result)
    }
	else{"No VMs with Connected Floppy Detected"}	
  }
}

$report += New-Object PSObject -Property @{
  "VM Attribute" = "VM's with Limits" 
  "Specification" = &{
    $result = Get-VMResourceConfiguration -VM $vms | Where-Object {$_.CpuLimitMhz -ne '-1'-Or $_.MemLimitMB -ne '-1'} | %{$_.VM.Name}
    if($result){
      [string]::Join(',',$result)
    }
	 else{"No VMs with Limits Detected"}
  }
}

$report += New-Object PSObject -Property @{
  "VM Attribute" = "VM's with Reservation" 
  "Specification" = &{
    $result = Get-VMResourceConfiguration -VM $vms | Where-Object {$_.CPUReservationMhz -ne '-0'-Or $_.MemReservationMB -ne '-0'} | %{$_.VM.Name}
    if($result){
      [string]::Join(',',$result)
    }
	 else{"No VMs with Reservation Detected"}
  }
}

$report += New-Object PSObject -Property @{
  "VM Attribute" = "VM's Created in Last 24 hours"
  "Specification" = &{
    $result = Get-VIEvent -MaxSamples ([int]::MaxValue) -Start (Get-Date).AddDays(-1) |
      where {$_.Gettype().Name -eq "VmCreatedEvent" -or $_.Gettype().Name -eq "VmBeingClonedEvent" -or $_.Gettype().Name -eq "VmBeingDeployedEvent"} |
      %{"{0} created by {1}" -f $_.VM.Name,$_.UserName}
    if($result){
      [string]::Join(',',$result)
    }
	else{"None"}
  }
}

$report += New-Object PSObject -Property @{
  "VM Attribute" = "VM's Deleted in Last 24 hours"
  "Specification" = &{
    $result = Get-VIEvent -MaxSamples ([int]::MaxValue) -Start (Get-Date).AddDays(-1) |
      where {$_.Gettype().Name -eq "VmRemovedEvent"} |
      %{"{0} created by {1}" -f $_.VM.Name,$_.UserName}
    if($result){
      [string]::Join(',',$result)
    }
	else{"None"}
  }
}

$report | Select "VM Attribute","Specification" | Export-Csv "C:\Aravind\Scripts\VMs.csv" -NoTypeInformation -UseCulture

&{foreach($row in (Import-Csv "C:\Aravind\Scripts\VMs.csv" -UseCulture)){
    $script:first = $true
    if($row."Specification" -match ','){
      ($row."Specification").Split(',') | %{
        New-Object PSObject -Property @{
          "VM Attribute" = &{
            if($script:first){
              $row."VM Attribute"
              $script:first = $false
            }
            else{""}
          }
          "Specification" = $_
        }
      }
    }
    else{$row}
}} | Select "VM Attribute","Specification" |
Export-Csv -Path $tempCSV -NoTypeInformation
Copy-CSVtoSheet $sheet $tempCSV

# Clean up workbook & close
$workbook.Sheets | where {$_.Name -like "Sheet*"} | %{$_.Delete()}
$nrSheets = $workbook.Sheets.Count
1..($nrSheets-1) |%{
	$workbook.Sheets.Item($nrSheets).Move($workbook.Sheets.Item($_))
}


$workbook.Sheets.Item(1).Select()
$workbook.SaveAs("C:\Scripts\$($vCenter)-vAssessmentReport-$($Date).xls")
$excelApp.Quit()
Stop-Process -Name "Excel"

# Close connections
Disconnect-VIServer -Server $vCenter -Confirm:$false

